// SPDX-License-Identifier: BSD-3-Clause
// Copyright (c) Contributors to the OpenEXR Project.

// This file is auto-generated by the cmake configure step

#ifndef INCLUDED_IEX_CONFIG_H
#define INCLUDED_IEX_CONFIG_H 1

#pragma once

//
//
// C++ namespace configuration / options

// automated formatting does not handle the cmake tags well
// clang-format off

//
// Current internal library namespace name
//
#define IEX_INTERNAL_NAMESPACE_CUSTOM 0
#define IEX_INTERNAL_NAMESPACE Iex_3_4

//
// Current public user namespace name
//

#define IEX_NAMESPACE_CUSTOM 0
#define IEX_NAMESPACE Iex

// clang-format on

#endif // INCLUDED_IEX_CONFIG_H
